from django import forms
from django.utils.translation import gettext as _

from django_ace import AceWidget
from django_select2.forms import ModelSelect2Widget
from material import Fieldset, Layout, Row

from aleksis.apps.kort.models import Card, CardLayout, CardLayoutMediaFile, CardPrinter


class CardForm(forms.ModelForm):
    printer = forms.ModelChoiceField(
        queryset=None,
        label=_("Card Printer"),
        help_text=_("Select a printer to directly print the newly issued card."),
        required=False,
    )

    class Meta:
        model = Card
        fields = ["person", "valid_until", "layout"]

        widgets = {
            "person": ModelSelect2Widget(
                search_fields=[
                    "first_name__icontains",
                    "last_name__icontains",
                    "short_name__icontains",
                ],
                attrs={"data-minimum-input-length": 0, "class": "browser-default"},
            ),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["layout"].required = True

        layouts = CardLayout.objects.all()
        if layouts.count() == 1:
            self.fields["layout"].initial = layouts.first()

        printers = CardPrinter.objects.all()
        self.fields["printer"].queryset = printers


class CardPrinterForm(forms.ModelForm):
    layout = Layout(
        Fieldset(_("Generic attributes"), "name", "location", "description"),
        Fieldset(
            _("Printer settings"), "cups_printer", "generate_number_on_server", "card_detector"
        ),
    )

    class Meta:
        model = CardPrinter
        fields = [
            "name",
            "location",
            "description",
            "cups_printer",
            "generate_number_on_server",
            "card_detector",
        ]


class PrinterSelectForm(forms.Form):
    layout = Layout("printer")
    printer = forms.ModelChoiceField(queryset=None, label=_("Card Printer"), required=True)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        printers = CardPrinter.objects.all()
        self.fields["printer"].queryset = printers
        if printers.count() == 1:
            self.fields["printer"].initial = printers.first()


class CardLayoutMediaFileForm(forms.ModelForm):
    layout = Layout(Row("media_file", "DELETE"))

    class Meta:
        model = CardLayoutMediaFile
        fields = ["media_file"]


class CardLayoutForm(forms.ModelForm):
    layout = Layout(Row("name"), Row("width", "height"), Row("template"), "css")

    template = forms.CharField(widget=AceWidget(mode="django"))
    css = forms.CharField(widget=AceWidget(mode="css"))

    class Meta:
        model = CardLayout
        fields = ["name", "template", "css", "width", "height"]


CardLayoutMediaFileFormSet = forms.inlineformset_factory(
    CardLayout, CardLayoutMediaFile, form=CardLayoutMediaFileForm
)
